import { tableSelect } from "@/components/TableSelect";
import { tableSort } from "@/components/TableSort";
import { tableSwitch } from "@/components/TableSwitch";
import { reactive, onMounted, ref, usePublicHooks, ElMessage, ElMessageBox, http, getGroupUrl, RegularVerification, verification, PaginationProps, hasAuth, getConfig } from "@/utils/importUsed"
// 接口列表实例
const UserUrl = ref(null)
const { switchStyle } = usePublicHooks();
// 获取当前板块接口列表
onMounted(async () => {
  UserUrl.value = await getGroupUrl(["User"]);

})
// 表单实例
const ruleFormRef = ref()
export function useUser() {
  // 搜索表单参数（需动态生成请求字段）
  const form = reactive({
    searchMain: "", // 搜索内容
    select: "", // 选择项
    date: "",
    sex: "",
  });
  // 性别映射表
  const sexMapList = [
    { id: 1, label: "男", type: "primary", effect: "light" },
    { id: 2, label: "女", type: "success", effect: "light" },
    { id: 0, label: "其他", type: "danger", effect: "light" }
  ]
  // 职级映射表
  const userRankMapList = [
    { id: 0, label: "K0", type: "warning", effect: "light" },
    { id: 1, label: "K1", type: "warning", effect: "light" },
    { id: 2, label: "K2", type: "warning", effect: "light" },
    { id: 3, label: "K3", type: "primary", effect: "light" },
    { id: 4, label: "K4", type: "primary", effect: "light" },
    { id: 5, label: "K5", type: "primary", effect: "light" },
    { id: 6, label: "K6", type: "success", effect: "light" },
    { id: 7, label: "K7", type: "success", effect: "light" },
    { id: 8, label: "K8", type: "success", effect: "light" },
    { id: 9, label: "K9", type: "success", effect: "light" }
  ]
  // 级联选择器参数
  const optionList = ref([
    {
      value: "resource",
      label: "Resource",
      children: [
        {
          value: "axure",
          label: "Axure Components"
        },
        {
          value: "sketch",
          label: "Sketch Templates"
        },
        {
          value: "docs",
          label: "Design Documentation"
        }
      ]
    }
  ]);
  // 表格数据列表
  const dataList:any = ref([
    { sort:0, id: 1, status:false,sex:1, name:'角色名称1', rank:3, dept:"行政部", post:["人事专员", "行政主管"], header:'kxs_new/KXS_ADMIN/2024-09-04/8ca30206-680f-48ef-aa07-8629cc83739fbf051c698ffdea8a.png',jsonData:{a:1}},
    { sort:1, id: 2, status:true,sex:0, name:'角色名称2', rank:9, dept:"运营部", post:["数据分析", "运营专员", "运营主管"], header:'kxs_new/KXS_ADMIN/2024-09-04/8ca30206-680f-48ef-aa07-8629cc83739fbf051c698ffdea8a.png',jsonData:{b:2} }
  ]);
  // 页面加载状态
  const loading = ref(false);
  // 表格分页选择器
  const pagination = reactive<PaginationProps>({
    total: 0,
    pageSize: 10,
    currentPage: 1,
    background: true
  });
  // 表格数据（根据字段生成）
  const columns: TableColumnList = [
    {
      type: "selection",
      width: 55,
      align: "left",
      hide: ({ checkList }) => !checkList.includes("勾选列")
    },
    {
      label: "序号",
      type: "index",
      width: 70,
      hide: ({ checkList }) => !checkList.includes("序号列")
    },
    {
      label: "角色ID",
      prop: "id",
      minWidth: 130
    },
    {
      label: "角色名称",
      prop: "name",
      minWidth: 130
    },
    {
      label: "职级",
      minWidth: 200,
      cellRenderer: ({ row, props }) => (
        <el-tag
          size={props.size}
          type={userRankMapList.find(item => item.id === row.rank)?.type}
          effect={userRankMapList.find(item => item.id === row.rank)?.effect}
        >
          {userRankMapList.find(item => item.id === row.rank)?.label}
          </el-tag>
      )
    },
    {
      label: "锁定状态",
      minWidth: 200,
      cellRenderer: (scope) => (
        tableSwitch({
          fields: 'status',
          scope,
          activeText:'未锁定',
          inactiveText:'已锁定',
          activeValue:true,
          inactiveValue:false,
          callback: onChange,
          isTip:true
        })
      )
    },
    {
      label: "性别",
      minWidth: 200,
      cellRenderer: (scope) => (
        tableSelect({
          fields: 'sex',
          dataList,
          scope,
          mapList: sexMapList,
          callback: changeSex,
          isTip:true
        })
      )
    },
    {
      label: "排序",
      minWidth: 200,
      cellRenderer: (scope) => (
       tableSort({
          fields: 'sort',
          dataList,
          scope,
          callback: changeSort,
          isTip:false
        })
      )
    },
    {
      label: "封面图",
      minWidth: 100,
      formatter: ({ header }) => {
        return (<div style="text-align: center;">
          <el-image
            fit="cover"
            preview-teleported={true}
            src={getConfig().UploadUrl + '/' + header}
            preview-src-list={Array.of(getConfig().UploadUrl + '/' + header)}
            class="w-[50px] h-[50px] align-middle"
          />
        </div>)
      }
    },
    {
      label: "部门",
      minWidth: 130,
      cellRenderer: ({ row, props }) => (
        <el-tag
          size={props.size}
          effect="plain"
        >
          { row.dept }
          </el-tag>
      )
    },
    {
      label: "岗位",
      minWidth: 130,
      cellRenderer: ({ row, props }) => (
        <div>
          {row.post.map((item: any) => (
            <el-tag size={props.size} type="success" effect="plain" class="mr-2">
              {item}
            </el-tag>
          ))}
        </div>
      )
    },
    {
      label: "操作",
      fixed: "right",
      width: 200,
      slot: "operation"
    }
  ];
  // 下拉框调整函数
  const changeSex = (row) => {
    const params = {
      id: row.Id,
      sex: row.sex
    };
    const { data }: any = http.request(
      UserUrl.User.userChangeSex.method,
      UserUrl.User.userChangeSex.url,
      params,
    );
    if (data.status != 1) {
      return
    } else {
      onSearch(ruleFormRef);
    }
    
  }
  // 表单顺序调整函数
  const changeSort = ({ row }) => {
    const params = {
      id: row.id,
      sort: row.sort
    };
    const { data }: any = http.request(
      UserUrl.User.userSearch.method,
      UserUrl.User.userSearch.url,
      params,
    );
    if (data.status != 1) {
      return
    } else {
      onSearch(ruleFormRef);
    }
  }
  // 表格开关
  function onChange({ row }, loadinClose) {
    if (row.status == 0) {
      handleUnlockUser(row, loadinClose)
    } else {
      handleLockUser(row, loadinClose)
    }
  }
    // 锁定用户
  async function handleLockUser(row, loadinClose) {
    const { status, msg }: any = await http.Request({
      method: UserUrl.llbMech.llbUserlockUser.method,
      url: UserUrl.llbMech.llbUserlockUser.url,
      params: {id: row.id,lockFlag:1}
    });
    if (status == 1) {
      ElMessage({
        message: "锁定成功",
        type: "success"
      });
    } else {
      ElMessageBox.alert(msg, "提示", {
        confirmButtonText: "关闭",
        type: "warning"
      });
      onSearch(ruleFormRef.value);
    }
    loadinClose()
  };
    // 解锁用户
  async function handleUnlockUser(row, loadinClose) {
    const { status, msg }: any = await http.Request({
      method: UserUrl.llbMech.llbUserlockUser.method,
      url: UserUrl.llbMech.llbUserlockUser.url,
      params: {id: row.id,lockFlag:0}
    });
    if (status == 1) {
      ElMessage({
        message: "解锁成功",
        type: "success"
      });
    } else {
      ElMessageBox.alert(msg, "提示", {
        confirmButtonText: "关闭",
        type: "warning"
      });
      onSearch(ruleFormRef.value);
    }
    loadinClose()
  };
  // 当前页数量切换
  function handleSizeChange(val: number) {
    if (typeof val === "number") {
      pagination.pageSize = val;
      onSearch(ruleFormRef);
    }
  }
  // 当前页码切换
  function handleCurrentChange(val: number) {
    console.log(`current page: ${val}`);
    if (typeof val === "number") {
      pagination.currentPage = val;
      onSearch(ruleFormRef);
    }
  }
  // 选择表格项
  function handleSelectionChange(val) {
    console.log(`SelectionChange: ${val}`);
    // selectList.value = val;
  }
  // 搜索列表
  async function onSearch(formEl) {
    // 表单校验拦截
    if (!formEl) return
    await formEl.validate((valid, fields) => {
      if (valid) {
        //表单校验成功回调
        console.log('submit!')
        // 状态调整为加载中
        loading.value = true;
        // 调用接口（需动态生成接口）
        const { data, other }: any = http.request(
          UserUrl.User.userSearch.method,
          UserUrl.User.userSearch.url,
          {
            ...form,
            page_size: pagination.pageSize,
            page_num: pagination.currentPage
          }
        );
        dataList.value = data;
        pagination.total = other.Count;
        setTimeout(() => {
          loading.value = false;
        }, 500);
      } else {
        //表单校验失败回调
        ElMessage({
          message: "请输入完整信息",
          type: "error"
        });
      }
    })
  }

  // 删除
  function handleDelete(row) {
    ElMessageBox.confirm(
      `是否删除该用户?  用户名称为:${row.name}`,
      "提示",
      {
        confirmButtonText: "删除",
        cancelButtonText: "取消",
        type: "warning"
      }
    ).then(async () => {
      const { status, info }: any = await http.Request({
        url: UserUrl.User.userDelete.method,
        method: UserUrl.User.userDelete.url,
        params: row.Id
      });
      if (status === "1") {
        ElMessage({
          message: "删除成功",
          type: "success"
        });
        onSearch(ruleFormRef.value);
      } else {
        ElMessageBox.alert(info, "提示", {
          confirmButtonText: "关闭",
          type: "warning"
        });
      }
    });
  }
  // 新增弹窗开启状态
  const addVisible = ref(false);
  function handleAdd() {
    addVisible.value = true;
  }
  // 修改弹窗开启状态
  const editVisible = ref(false);
  // 修改弹窗数据
  const editFormData = ref({});
  function handleUpdate(row) {
    editVisible.value = true;
    // 表格数据赋值
    editFormData.value = row;
  }

  // 表单校验规则
  const rules = reactive({
    searchMain: [
      // 必传(字段名动态生成)
      { required: true, message: '请输入搜索内容', trigger: 'blur' },
    ],
    select: [
      // 必传
      { required: true, message: '请输入选择项', trigger: 'blur' },
      // 长度
      { min: 3, max: 5, message: '长度为3-5', trigger: 'blur' },
      // 时间类型必填
      {
        type: 'date',
        required: true,
        message: '请选择正确时间类型',
        trigger: 'blur',
      },
      // 封装正则
      {
        validator: (rule: any, value: any, callback: any) => {
          verification.verificationName(value, callback)
        }, trigger: 'blur'
      },
      // 自定义正则
      {
        validator: (rule: any, value: any, callback: any) => {
          // 自定义正则替换
          const regular = /^((13[0-9])|(14(0|[5-7]|9))|(15([0-3]|[5-9]))|(16(2|[5-7]))|(17[0-8])|(18[0-9])|(19([0-3]|[5-9])))\d{8}$/;
          RegularVerification(regular, value, callback)
        }, trigger: 'blur'
      },
    ],
  })
  return {
    form,
    loading,
    columns,
    dataList,
    pagination,
    onSearch,
    handleSizeChange,
    handleCurrentChange,
    handleSelectionChange,
    handleAdd,
    handleUpdate,
    handleDelete,
    addVisible,
    editVisible,
    editFormData,
    optionList,
    ruleFormRef,
    rules,
    sexMapList
  };
}
